"""

epyk_npm.exe
"""


def install():
  """
  Install the package following the below route:
    - Test if the package existing in NPM
    - Test if the package is availalbe on CDNJS
  """


def add():
  """
  Add an extra external package in the project repository
  """


def info():
  """
  Get package information from NPM repository and check if the current version in the project is the same
  """


def cdnjs():
  """
  Get package information from CDNJS
  """
