"""

epyk_export.exe
"""


def angular():
  """
  Generate an Angular Application from the Epyk page
  """


def vue():
  pass


def react():
  pass


def node():
  pass


def jupyter():
  pass


def deno():
  pass


def html():
  pass


def serve():
  pass
