"""
Main command lines for Epyk

epyk.exe
"""


def new():
  """
  Create a new Epyk Structure.

  THis structure should be easy to integrate any Python exnsting project:

  The structure can be the ones below
  ui/
    pages/
    views/
    packages/
    settings.py
    transpile.py
    serve.py
  """


def page():
  """
  Create a new page in the current project

  basically it can be something like the below:

  def page():
    '''
    '''

  """


def transpile():
  """
  Transpile to HTML a report
  """
